@@ -5,8 +5,8 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 0.1
-;; Package-Requires: ((emacs "27.1") (vertico "1.4"))
+;; Version: 1.6
+;; Package-Requires: ((emacs "27.1") (compat "29.1.4.4") (vertico "1.6"))
 ;; Homepage: https://github.com/minad/vertico
 
 ;; This file is part of GNU Emacs.
@@ -49,11 +49,12 @@
 ;; Temporary toggling between the different display modes is possible.
 ;; The following keys are bound in the `vertico-multiform-map'.
 ;;
-;;   M-V -> `vertico-multiform-vertical'
-;;   M-G -> `vertico-multiform-grid'
+;;   M-B -> `vertico-multiform-buffer'
 ;;   M-F -> `vertico-multiform-flat'
+;;   M-G -> `vertico-multiform-grid'
 ;;   M-R -> `vertico-multiform-reverse'
 ;;   M-U -> `vertico-multiform-unobtrusive'
+;;   M-V -> `vertico-multiform-vertical'
 ;;
 ;;; Code:
 
@@ -106,11 +107,11 @@ The keys in LIST can be symbols or regexps."
 (defun vertico-multiform--setup ()
   "Enable modes at minibuffer setup."
   (let ((cat (completion-metadata-get
-              (completion-metadata
-               (buffer-substring (minibuffer-prompt-end)
-                                 (max (minibuffer-prompt-end) (point)))
-               minibuffer-completion-table
-               minibuffer-completion-predicate)
+              (completion-metadata (buffer-substring-no-properties
+                                    (minibuffer-prompt-end)
+                                    (max (minibuffer-prompt-end) (point)))
+                                   minibuffer-completion-table
+                                   minibuffer-completion-predicate)
               'category))
         (exit (make-symbol "vertico-multiform--exit"))
         (depth (recursion-depth))
@@ -139,11 +140,12 @@ The keys in LIST can be symbols or regexps."
 
 (defvar-keymap vertico-multiform-map
   :doc "Additional keymap activated in multiform mode."
-  "M-V" #'vertico-multiform-vertical
-  "M-G" #'vertico-multiform-grid
+  "M-B" #'vertico-multiform-buffer
   "M-F" #'vertico-multiform-flat
+  "M-G" #'vertico-multiform-grid
   "M-R" #'vertico-multiform-reverse
-  "M-U" #'vertico-multiform-unobtrusive)
+  "M-U" #'vertico-multiform-unobtrusive
+  "M-V" #'vertico-multiform-vertical)
 
 ;;;###autoload
 (define-minor-mode vertico-multiform-mode
@@ -154,9 +156,9 @@ The keys in LIST can be symbols or regexps."
   (when vertico-multiform--stack
     (warn "vertico-multiform state is inconsistent")
     (setq vertico-multiform--stack nil))
-  (if vertico-multiform-mode
-      (add-to-list 'minor-mode-map-alist `(vertico--input . ,vertico-multiform-map))
-    (setq minor-mode-map-alist (delete `(vertico--input . ,vertico-multiform-map) minor-mode-map-alist))))
+  (cl-callf2 rassq-delete-all vertico-multiform-map minor-mode-map-alist)
+  (when vertico-multiform-mode
+    (push `(vertico--input . ,vertico-multiform-map) minor-mode-map-alist)))
 
 (cl-defmethod vertico--advice (&context (vertico-multiform-mode (eql t)) &rest app)
   (unwind-protect
@@ -188,8 +190,8 @@ ARG can be nil, t, -1, 1 or toggle."
         (push not-mode (car vertico-multiform--stack))))))
 
 (defvar vertico-multiform--display-modes
-  '(vertico-unobtrusive-mode vertico-flat-mode
-    vertico-grid-mode vertico-reverse-mode))
+  '(vertico-unobtrusive-mode vertico-flat-mode vertico-grid-mode
+    vertico-reverse-mode vertico-buffer-mode))
 (defvar-local vertico-multiform--display-last nil)
 
 (defun vertico-multiform-vertical (&optional mode)
@@ -214,6 +216,7 @@ MODE defaults to the vertical display."
      (interactive)
      (vertico-multiform-vertical ',(intern (format "vertico-%s-mode" name)))))
 
+(vertico-multiform--define-display-toggle buffer)
 (vertico-multiform--define-display-toggle grid)
 (vertico-multiform--define-display-toggle flat)
 (vertico-multiform--define-display-toggle reverse)
